
#include "BBCBillboardCloud.h"
#include "BBCBillboard.h"

namespace BBC {

BillboardCloud::BillboardCloud() 
{
	mEntity = new Entity();
}

BillboardCloud::~BillboardCloud() 
{
	for (unsigned int iBillboard = 0; iBillboard < getNumBillboards(); iBillboard++)
	{
		Billboard *billboard = getBillboard(iBillboard);
		delete billboard;
	}

	delete mEntity;
}

vector<Billboard*>* BillboardCloud::getBillboardList() {
  return &mBillboardList;
}
void BillboardCloud::setBillboardList(vector<Billboard*> &value) {
  mBillboardList = value;
}

unsigned int BillboardCloud::getNumBillboards()
{
	return (unsigned int)(mBillboardList.size());
}

void BillboardCloud::addBillboard(Billboard *value) 
{
	value->setBillboardHandle((unsigned int)(mBillboardList.size()));	
	mBillboardList.push_back(value);
}

void BillboardCloud::removeBillboard(unsigned int value) 
{
	Billboard *billboard = mBillboardList[value];
	mBillboardList.erase(mBillboardList.begin() + value);
	delete billboard;
}

Billboard* BillboardCloud::getBillboard(unsigned int value) 
{
	return mBillboardList[value];
}

Entity* BillboardCloud::getEntity() 
{
	return mEntity;
}

void BillboardCloud::setEntity(Entity *value) 
{
	mEntity = value;
}

void BillboardCloud::generateBillboardCloud(bool mergeBillboards)
{
	unsigned int numGeneratedBillboardClouds = 0;
	for (unsigned int iBillboard = 0; iBillboard < getNumBillboards(); iBillboard++)
	{
		Billboard *billboard = this->getBillboard(iBillboard);		
		EntityCluster *entityCluster = billboard->getBillboardClusterData()->getEntityCluster();

		if (entityCluster->getNumEntitiesClusterData() > 0)
		{		
			SubEntity *subEntity;			
			if (mergeBillboards)
			{
				subEntity = mEntity->getSubEntity(0);
				unsigned int offset = subEntity->getNumFaces() * 3;
				Ogre::Vector3 indicesA = Ogre::Vector3(offset, offset + 1, offset + 2);
				subEntity->addFaceVerticesIDs(indicesA);
				offset = subEntity->getNumFaces()*3;
				Ogre::Vector3 indicesB = Ogre::Vector3(offset, offset + 1, offset + 2);
				subEntity->addFaceVerticesIDs(indicesB);
			}
			else
			{				
				// The entity has by default one subentity, if we need more
				// we have to create them.
				if (numGeneratedBillboardClouds > 0)
				{
					mEntity->createSubEntity();
				}
				Ogre::Vector3 indices = Ogre::Vector3(0,1,2);
				subEntity = mEntity->getSubEntity(numGeneratedBillboardClouds);
				subEntity->addTextureCoordSet(2);				
				subEntity->addFaceVerticesIDs(indices);
				subEntity->addFaceVerticesIDs(indices + Ogre::Vector3(3,3,3));
			}
			
			UniqueVertex vFaceA1,vFaceA2,vFaceA3;
			// Face A of the billboard quad
			vFaceA1.position = billboard->getBillboardClusterData()->getQuadTopLeftCorner();
			vFaceA1.normal = billboard->getBillboardClusterData()->getNormal();
			vFaceA1.uv[0] = Ogre::Vector3(0,0,0);
			vFaceA2.position = billboard->getBillboardClusterData()->getQuadTopRightCorner();
			vFaceA2.normal = billboard->getBillboardClusterData()->getNormal();
			vFaceA2.uv[0] = Ogre::Vector3(1,0,0);
			vFaceA3.position = billboard->getBillboardClusterData()->getQuadBottomRightCorner();
			vFaceA3.normal = billboard->getBillboardClusterData()->getNormal();
			vFaceA3.uv[0] = Ogre::Vector3(1,1,0);

			// Face B of the billboard quad
			UniqueVertex vFaceB1,vFaceB2,vFaceB3;
			vFaceB1.position = billboard->getBillboardClusterData()->getQuadBottomRightCorner();
			vFaceB1.normal = billboard->getBillboardClusterData()->getNormal();
			vFaceB1.uv[0] = Ogre::Vector3(1,1,0);
			vFaceB2.position = billboard->getBillboardClusterData()->getQuadBottomLeftCorner();
			vFaceB2.normal = billboard->getBillboardClusterData()->getNormal();
			vFaceB2.uv[0] = Ogre::Vector3(0,1,0);
			vFaceB3.position = billboard->getBillboardClusterData()->getQuadTopLeftCorner();
			vFaceB3.normal = billboard->getBillboardClusterData()->getNormal();
			vFaceB3.uv[0] = Ogre::Vector3(0,0,0);

			subEntity->addUniqueVertex(vFaceA1);
			subEntity->addUniqueVertex(vFaceA2);
			subEntity->addUniqueVertex(vFaceA3);
			subEntity->addUniqueVertex(vFaceB1);
			subEntity->addUniqueVertex(vFaceB2);
			subEntity->addUniqueVertex(vFaceB3);

			numGeneratedBillboardClouds++;
		}
	}
}

}
