
#include "BBCBillboardCloudUVMapper.h"
#include "BBCBillboardCloud.h"
#include "BBCEntityCluster.h"
#include "BBCBillboardClusterData.h"

namespace BBC {

void BillboardCloudUVMapper::initialize() 
{
	IMG::Box2d *box = new IMG::Box2d;
	IMG::NodeBsp *node = new IMG::NodeBsp;
	box->SetBoundBox(0, 0, mTextureAtlasWidth, mTextureAtlasHeight);
	node->SetBound(box);
	mBsp.SetRoot(node);	
}

void BillboardCloudUVMapper::setTextureWidth(unsigned short int width) 
{
	mTextureWidth = width;
}

unsigned short int BillboardCloudUVMapper::getTextureWidth() 
{
	return mTextureWidth;
}

void BillboardCloudUVMapper::setTextureHeight(unsigned short int height) 
{
	mTextureHeight = height;
}

unsigned short BillboardCloudUVMapper::getTextureHeight() 
{
	return mTextureHeight;
}

void BillboardCloudUVMapper::setTextureAtlasWidth(unsigned short int width) 
{
	mTextureAtlasWidth = width;
}

unsigned short int BillboardCloudUVMapper::getTextureAtlasWidth() 
{
	return mTextureAtlasWidth;
}

void BillboardCloudUVMapper::setTextureAtlasHeight(unsigned short int height) 
{
	mTextureAtlasHeight = height;
}

unsigned short BillboardCloudUVMapper::getTextureAtlasHeight() 
{
	return mTextureAtlasHeight;
}

void BillboardCloudUVMapper::generateTextureAtlasTextureCoords(bool mergeBillboards) 
{	

	for (unsigned int iBillboard = 0; iBillboard < mBillboardCloud->getNumBillboards(); iBillboard++)
	{
		Ogre::LogManager::getSingleton().logMessage("id:" + Ogre::StringConverter::toString(iBillboard));
		BillboardClusterData *billboardClusterData = mBillboardCloud->getBillboard(iBillboard)->getBillboardClusterData();		
		
		if (billboardClusterData != NULL)
		{
			Ogre::LogManager::getSingleton().logMessage("billboardData");
			EntityCluster *entityCluster = billboardClusterData->getEntityCluster();
			if (entityCluster->getNumEntitiesClusterData() > 0)
			{			
				Ogre::LogManager::getSingleton().logMessage("entityCluster");
				
				IMG::NodeBsp *nodeDest;
				IMG::Cluster cluster;
				Ogre::LogManager::getSingleton().logMessage("w:" + Ogre::StringConverter::toString(mTextureWidth));
				Ogre::LogManager::getSingleton().logMessage("h:" + Ogre::StringConverter::toString(mTextureHeight));
				nodeDest = mBsp.Insert(mTextureWidth,mTextureHeight,iBillboard);
				
				cluster.SetBspNodeBound(nodeDest->GetBound());
				//cluster.Finish();
				mClusterList.Insert(&cluster);				
			}
		}
	}
	
	//mBsp.Print();		

	unsigned int numGeneratedBillboardClouds = 0;
	bool enabledTextureCoordSet = false;
	unsigned int iUniqueVertex = 0;

	for (unsigned int iBillboard = 0; iBillboard < mBillboardCloud->getNumBillboards(); iBillboard++)
	{							
		Billboard *billboard = mBillboardCloud->getBillboard(iBillboard);		
		EntityCluster *entityCluster = billboard->getBillboardClusterData()->getEntityCluster();

		if (entityCluster->getNumEntitiesClusterData() > 0)
		{	
			IMG::Cluster *cluster = mClusterList.Get(numGeneratedBillboardClouds);		
			IMG::Box2d *box = cluster->GetBspNodeBound();
			Ogre::Vector2 max = box->GetMaximum();
			Ogre::Vector2 min = box->GetMinimum();
			Ogre::LogManager::getSingleton().logMessage("Max:" + Ogre::StringConverter::toString(Ogre::Vector3(max[0],max[1],0)) + 
				"\tMin:" + Ogre::StringConverter::toString(Ogre::Vector3(min[0],min[1],0)) + 
				"\tSize(" + Ogre::StringConverter::toString(max[0]-min[0]) +
				"," + Ogre::StringConverter::toString(max[1]-min[1]) + ")");


			SubEntity *subEntity;												
			
			Ogre::Vector2 texCoords01,texCoords02,texCoords03,texCoords04;
			texCoords01[0] = min[0] / mTextureAtlasWidth;
			texCoords01[1] = min[1] / mTextureAtlasHeight;

			texCoords02[0] = max[0] / mTextureAtlasWidth;
			texCoords02[1] = min[1] / mTextureAtlasHeight;

			texCoords03[0] = max[0] / mTextureAtlasWidth;
			texCoords03[1] = max[1] / mTextureAtlasHeight;

			texCoords04[0] = min[0] / mTextureAtlasWidth;
			texCoords04[1] = max[1] / mTextureAtlasHeight;
			
			if (!mergeBillboards)
			{
				subEntity = mBillboardCloud->getEntity()->getSubEntity(numGeneratedBillboardClouds);
				subEntity->addTextureCoordSet(2);
				subEntity->enableVertexColours(true);
				// Face A
				//(0,0,0);
				//(1,0,0);
				//(1,1,0);

				subEntity->setTexCoord(0, subEntity->getNumTexCoordSets()-1, Ogre::Vector3(texCoords01[0],texCoords01[1], 0));
				subEntity->setTexCoord(1, subEntity->getNumTexCoordSets()-1, Ogre::Vector3(texCoords02[0],texCoords02[1], 0));
				subEntity->setTexCoord(2, subEntity->getNumTexCoordSets()-1, Ogre::Vector3(texCoords03[0],texCoords03[1], 0));

				Ogre::ColourValue subEntityColour01 = Ogre::ColourValue(0, texCoords01[0],texCoords01[1]);
				Ogre::ColourValue subEntityColour02 = Ogre::ColourValue(0, texCoords02[0],texCoords02[1]);
				Ogre::ColourValue subEntityColour03 = Ogre::ColourValue(0, texCoords03[0],texCoords03[1]);

				subEntity->setVertexColour(0, subEntityColour01.getAsRGBA());
				subEntity->setVertexColour(1, subEntityColour02.getAsRGBA());
				subEntity->setVertexColour(2, subEntityColour03.getAsRGBA());

				// Face B
				//(1,1,0);
				//(0,1,0);
				//(0,0,0);
				subEntity->setTexCoord(3, subEntity->getNumTexCoordSets()-1, Ogre::Vector3(texCoords03[0],texCoords03[1],0));
				subEntity->setTexCoord(4, subEntity->getNumTexCoordSets()-1, Ogre::Vector3(texCoords04[0],texCoords04[1],0));
				subEntity->setTexCoord(5, subEntity->getNumTexCoordSets()-1, Ogre::Vector3(texCoords01[0],texCoords01[1],0));

				Ogre::ColourValue subEntityColour04 = Ogre::ColourValue(0, texCoords03[0],texCoords03[1]);
				Ogre::ColourValue subEntityColour05 = Ogre::ColourValue(0, texCoords04[0],texCoords04[1]);
				Ogre::ColourValue subEntityColour06 = Ogre::ColourValue(0, texCoords01[0],texCoords01[1]);

				subEntity->setVertexColour(3, subEntityColour04.getAsRGBA());
				subEntity->setVertexColour(4, subEntityColour05.getAsRGBA());
				subEntity->setVertexColour(5, subEntityColour06.getAsRGBA());

			}
			else
			{
				subEntity = mBillboardCloud->getEntity()->getSubEntity(0);				
				if (!enabledTextureCoordSet)
				{
					subEntity->addTextureCoordSet(2);
					subEntity->enableVertexColours(true);
					enabledTextureCoordSet = true;
				}
				
				// Face A
				//(0,0,0);
				//(1,0,0);
				//(1,1,0);

				subEntity->setTexCoord(iUniqueVertex, subEntity->getNumTexCoordSets()-1, Ogre::Vector3(texCoords01[0],texCoords01[1],0));
				subEntity->setTexCoord(iUniqueVertex+1, subEntity->getNumTexCoordSets()-1, Ogre::Vector3(texCoords02[0],texCoords02[1],0));
				subEntity->setTexCoord(iUniqueVertex+2, subEntity->getNumTexCoordSets()-1, Ogre::Vector3(texCoords03[0],texCoords03[1],0));

				Ogre::ColourValue subEntityColour01 = Ogre::ColourValue(0, texCoords01[0],texCoords01[1]);
				Ogre::ColourValue subEntityColour02 = Ogre::ColourValue(0, texCoords02[0],texCoords02[1]);
				Ogre::ColourValue subEntityColour03 = Ogre::ColourValue(0, texCoords03[0],texCoords03[1]);

				subEntity->setVertexColour(iUniqueVertex, subEntityColour01.getAsRGBA());
				subEntity->setVertexColour(iUniqueVertex+1, subEntityColour02.getAsRGBA());
				subEntity->setVertexColour(iUniqueVertex+2, subEntityColour03.getAsRGBA());


				// Face B
				//(1,1,0);
				//(0,1,0);
				//(0,0,0);
				subEntity->setTexCoord(iUniqueVertex+3, subEntity->getNumTexCoordSets()-1, Ogre::Vector3(texCoords03[0],texCoords03[1],0));
				subEntity->setTexCoord(iUniqueVertex+4, subEntity->getNumTexCoordSets()-1, Ogre::Vector3(texCoords04[0],texCoords04[1],0));
				subEntity->setTexCoord(iUniqueVertex+5, subEntity->getNumTexCoordSets()-1, Ogre::Vector3(texCoords01[0],texCoords01[1],0));

				Ogre::ColourValue subEntityColour04 = Ogre::ColourValue(0, texCoords03[0],texCoords03[1]);
				Ogre::ColourValue subEntityColour05 = Ogre::ColourValue(0, texCoords04[0],texCoords04[1]);
				Ogre::ColourValue subEntityColour06 = Ogre::ColourValue(0, texCoords01[0],texCoords01[1]);

				subEntity->setVertexColour(iUniqueVertex+3, subEntityColour04.getAsRGBA());
				subEntity->setVertexColour(iUniqueVertex+4, subEntityColour05.getAsRGBA());
				subEntity->setVertexColour(iUniqueVertex+5, subEntityColour06.getAsRGBA());

				iUniqueVertex = iUniqueVertex + 6;
			}
			numGeneratedBillboardClouds++;
		}
	}
	
}

void BillboardCloudUVMapper::setBillboardCloud(BillboardCloud *billboardCloud) 
{
	mBillboardCloud = billboardCloud;
}

BillboardCloud * BillboardCloudUVMapper::getBillboardCloud() 
{
	return mBillboardCloud;
}

BillboardCloudUVMapper::BillboardCloudUVMapper() 
{
}

BillboardCloudUVMapper::~BillboardCloudUVMapper() 
{
}

}
