#include <LBBC.h>
#include <LBBCPrerequisites.h>

struct CommandLineOptions
{
	Ogre::String cfg;
};

int findCommandLineOpts(int numargs, char** argv, Ogre::UnaryOptionList& unaryOptList, 
						Ogre::BinaryOptionList& binOptList)
{
    int startIndex = 1;
    for (int i = 1; i < numargs; ++i)
    {
		Ogre::String tmp(argv[i]);
		if (Ogre::StringUtil::startsWith(tmp, "-"))
        {
			Ogre::UnaryOptionList::iterator ui = unaryOptList.find(argv[i]);
            if(ui != unaryOptList.end())
            {
                ui->second = true;
                ++startIndex;
                continue;
            }
			Ogre::BinaryOptionList::iterator bi = binOptList.find(argv[i]);
            if(bi != binOptList.end())
            {
                bi->second = argv[i+1];
                startIndex += 2;
                ++i;
                continue;
            }

            // Invalid option
            cout<<"Invalid option "<<tmp<<endl;

        }
    }
    return startIndex;
}

CommandLineOptions parseArgs(int numArgs, char **args)
{
    CommandLineOptions opts;

	Ogre::UnaryOptionList unOpt;
	Ogre::BinaryOptionList binOpt;

    binOpt["-cfg"] = "";
    
	int startIndex = findCommandLineOpts(numArgs, args, unOpt, binOpt);
	Ogre::UnaryOptionList::iterator ui;
	Ogre::BinaryOptionList::iterator bi;
	
    bi = binOpt.find("-cfg");
    if (!bi->second.empty())
    {
		opts.cfg = bi->second;	
    }	

    return opts;
}

void help()
{
	cout<<"Please specify the parameter: -cfg sample.cfg"<<endl;
}

int main(int numargs, char** args)
{	
	/*
	if (numargs < 2)
    {
        help();
        return -1;
    }
    */

	CommandLineOptions opts = parseArgs(numargs, args);

	LBBC::Manager* manager = new LBBC::Manager();
	LBBC::Manager::getSingleton().initialize();	

	manager->runSample(Ogre::String(opts.cfg));	

	LBBC::Manager::getSingleton().shutdown();
	delete manager;	

	return 0;
}

